Package: specr
Title: Conducting and Visualizing Specification Curve Analyses
Version: 1.0.0
Authors@R: c(
    person(given = "Philipp K.",
           family = "Masur",
           role = c("aut", "cre"),
           email = "phil.masur@gmail.com",
           comment = c(ORCID = "https://orcid.org/0000-0003-3065-7305")),
    person(given = "Michael",
           family = "Scharkow",
           role = c("aut"),
           email = "michael@underused.org")
           )
Description: Provides utilities for conducting specification curve analyses (Simonsohn, Simmons & Nelson (2020, <doi: 10.1038/s41562-020-0912-z>) or multiverse analyses (Steegen, Tuerlinckx, Gelman & Vanpaemel, 2016, <doi: 10.1177/1745691616658637>) including functions to setup, run, evaluate, and plot all specifications.
License: GPL-3
URL: https://masurp.github.io/specr/, https://github.com/masurp/specr
BugReports: https://github.com/masurp/specr/issues
Depends: R (>= 3.5.0)
Imports: broom, cowplot, dplyr, furrr, future, ggplot2, ggraph, glue,
        igraph, lifecycle, lme4, magrittr, methods, parallelly, purrr,
        rlang, stringr, tibble, tidyr
Suggests: broom.mixed, gapminder, ggridges, knitr, lavaan, testthat,
        tidyverse, performance, rmarkdown
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-01-20 13:17:23 UTC; philippmasur
Author: Philipp K. Masur [aut, cre] (<https://orcid.org/0000-0003-3065-7305>),
  Michael Scharkow [aut]
Maintainer: Philipp K. Masur <phil.masur@gmail.com>
Repository: CRAN
Date/Publication: 2023-01-20 13:50:02 UTC
Built: R 4.6.0; ; 2026-04-30 17:10:20 UTC; unix
