# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,specr.object)
S3method(as.data.frame,specr.setup)
S3method(as_tibble,specr.object)
S3method(as_tibble,specr.setup)
S3method(plot,specr.object)
S3method(plot,specr.setup)
S3method(print,specr.object)
S3method(print,specr.setup)
S3method(summary,specr.object)
S3method(summary,specr.setup)
export("%>%")
export(as_tibble)
export(icc_specs)
export(plot_choices)
export(plot_curve)
export(plot_decisiontree)
export(plot_grid)
export(plot_samplesizes)
export(plot_specs)
export(plot_summary)
export(plot_variance)
export(run_specs)
export(setup)
export(setup_specs)
export(specr)
export(summarise_specs)
import(dplyr)
import(ggplot2)
importFrom(cowplot,plot_grid)
importFrom(furrr,future_pmap)
importFrom(future,multisession)
importFrom(future,nbrOfWorkers)
importFrom(future,plan)
importFrom(glue,glue)
importFrom(igraph,graph_from_data_frame)
importFrom(lme4,VarCorr)
importFrom(lme4,lmer)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(parallelly,availableCores)
importFrom(purrr,cross)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_df)
importFrom(purrr,pmap)
importFrom(purrr,reduce)
importFrom(rlang,.data)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(stats,formula)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,nobs)
importFrom(stats,quantile)
importFrom(stats,start)
importFrom(stats,vcov)
importFrom(stringr,str_glue)
importFrom(stringr,str_remove_all)
importFrom(tibble,as_tibble)
importFrom(tibble,lst)
importFrom(tibble,tibble)
importFrom(tidyr,expand_grid)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,stack)
